=begin
(c) TIG 2010
Permission to use, copy, modify, and distribute this software for 
any purpose and without fee is hereby granted, provided that the above
copyright notice appear in all copies.
THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR
IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
___________________________________________________________________________

'radiallines.rb' >>> ../Plugins/ folder
___________________________________________________________________________
Usage:
Select one guide-point [cpoint] and some edges and/or faces
then run Plugins menu 'Radial Lines...' sub-menu items...
 Lines -               draws a line from the cpoint to each vertex.
 Lines [layered] -     draws a line from the cpoint to each vertex on Layer.
 Guides -              draws a guide-line from the cpoint to each vertex.
 Guides with Lengths - draws a guide-line from the cpoint to each vertex, 
                       and adds central text to it showing its length, 
                       in current units.
___________________________________________________________________________
Donations:
by PayPal to info @ revitrev.org
___________________________________________________________________________
Version:
1.0 20100923 First release.
1.1 20100923 Line [layered] option added.
___________________________________________________________________________
=end
###
require 'sketchup.rb'
###
def radiallines(as_cline=true, with_text=true, layered=false)
  model=Sketchup.active_model
  ss=model.selection
  sa=ss.to_a
  cp=nil
  vs=[]
  sa.each{|e|
    cp=e if e.class==Sketchup::ConstructionPoint
    e.vertices.each{|v|vs << v if not vs.include?(v)} if e.class==Sketchup::Face
    if e.class==Sketchup::Edge
      vs << e.start if not vs.include?(e.start)
      vs << e.end if not vs.include?(e.end)
    end#if
  }
  if not cp
    UI.messagebox("Selection must contain a Guide-Point !")
    return nil
  end#if
  if not vs[0]
    UI.messagebox("Selection must contain a Vertex !")
    return nil
  end#if
  model.start_operation("Radial Lines")
  ###
  #ss.clear
  ###
  ents=model.active_entities
  base=cp.position
  ###
  if as_cline or layered
    layer1=model.layers.add("RADL-LINE")
    layer1.visible=true
  end#if
  if as_cline
    cb=ents.add_cpoint(base)
    cb.layer=layer1
  end#if
  ###
  if with_text
    layer2=model.layers.add("RADL-LINE-TEXT")
    layer2.visible=true
  end#if
  points=[]
  vs.each{|v|points << v.position}
  points.each{|p|
    if as_cline
      pc=ents.add_cpoint(p)
      pc.layer=layer1
      rl=ents.add_cline(base,p)
      rl.layer=layer1
    else
      rl=ents.add_line(base,p)
      rl.layer=layer1 if layered
    end#if
    if with_text
      dis=base.distance(p)
      mid=base.offset(base.vector_to(p), dis/2)
      text=dis.to_l.to_s
      txt=ents.add_text(text,mid,[0,0,0])
      txt.layer=layer2
      txt.leader_type=0
      txt.arrow_type=0
      txt.display_leader=false
    end#if
  }
  ###
  model.commit_operation
end
### menu
if not file_loaded?(File.basename(__FILE__))
  submenu=UI.menu("Plugins").add_submenu("Radial Lines...")
  submenu.add_item("Lines"){radiallines(false,false,false)}
  submenu.add_item("Lines [layered]"){radiallines(false,false,true)}
  submenu.add_item("Guides"){radiallines(true,false,false)}
  submenu.add_item("Guides with Lengths"){radiallines(true,true,false)}
end#if
file_loaded(File.basename(__FILE__))
###
